/************************************************************************
 *   IRC - Internet Relay Chat, doc/example_module.c
 *   Copyright (C) 2001 Hybrid Development Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_grated.c,v 1.5 2002/12/08 09:28:49 jeremy Exp $
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"     /* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "ircd_defs.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "chanfilter.h"


static void m_grated(struct Client *client_p, struct Client *source_p, int parc, char *parv[]);

struct Message grated_msgtab = {
  "GRATED", 0, 0, 2, 2, MFLG_SLOW, 0,
  {m_ignore, m_grated, m_grated, m_grated}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&grated_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&grated_msgtab);
}

const char *_version = "$Revision: 1.5 $";
#endif

/* m_grated
 *     parv[0] = sender prefix
 *     parv[1] = channel
 */
static void m_grated(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
  struct Channel *chptr;
  FILE *fp;
  char buf[BUFSIZE];
  char *tmp;
  char words[BUFSIZE];
  char *ch = parv[1];
  dlink_list *list;
  dlink_node *ptr;
  dlink_node *tptr;
  struct BadWords *badword;
  words[0] = '\0';

  if(parv[1][0] == '-')
    {
      tmp = strchr(ch, '-');
      *tmp = '\0';
      tmp++;
      if((chptr = hash_find_channel(tmp)) == NULL)
        {
          sendto_one(source_p, "%s :No such channel", tmp);
          return;
        }
      if(!IsMember(source_p, chptr))
        {
          sendto_one(source_p, form_str(ERR_NOTONCHANNEL), me.name, parv[0], tmp);
          return;
        }
      if(!is_chan_op(chptr, source_p))
        {
          sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED), me.name, parv[0], tmp);
          return;
        }
      if(!ConfigChannel.use_badwords)
        {
          sendto_one(source_p, "%s :+G Not Enabled", tmp);
          return;
        }

        list = &chptr->badwordlist;

        DLINK_FOREACH_SAFE(ptr, tptr, list->head)
          {
            badword = ptr->data;

            sendto_channel_local(ONLY_SERVERS, chptr, ":%s!%s@%s MODE %s :-G %s", source_p->name, source_p->username, source_p->host, chptr->chname, badword->word);
            sendto_server(NULL, chptr, CAP_BAD, NOCAPS, ":%s!%s@%s MODE %s :-G %s", source_p->name, source_p->username, source_p->host, chptr->chname, badword->word);

            /*memcounts.badwords--;
            memcounts.badwords_size = memcounts.badwords_size - sizeof(ptr);*/

            MyFree(badword->word);
            MyFree(badword->who);
            BlockHeapFree(badword_heap, badword);

            if(chptr->num_words > 0)
              chptr->num_words--;
            else
              chptr->num_words = 0;

            dlinkDestroy(ptr, list);
          }
      sendto_one(source_p, "%s :Flushing Filter", tmp);
    }
  else
    {
      if((chptr = hash_find_channel(ch)) == NULL)
        {
          sendto_one(source_p, "%s :No such channel", parv[1]);
          return;
        }
      if(!IsMember(source_p, chptr))
        {
          sendto_one(source_p, form_str(ERR_NOTONCHANNEL), me.name, parv[0],
                         parv[1]);
          return;
        }
      if(!is_chan_op(chptr, source_p))
        {
          sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
                       me.name, parv[0], parv[1]);
          return;
        }
      
      if(!ConfigChannel.use_badwords)
        {
          sendto_one(source_p, "+G Not Enabled");
          return;
        }

      if((fp = fopen(ConfigFileEntry.filter_file, "r")) == NULL)
        {
          sendto_realops_flags(UMODE_DEBUG, L_ALL, "Failed Opening %s", ConfigFileEntry.filter_file);
          return;
        }

      while (fgets(buf, BUFSIZE, fp))
        {
          if((buf[0] != ';') && (buf[0] != '#') && (buf[0] != '\n') && (buf[0] != '\r'))
            {
              tmp = strchr(buf, '\n');
              if(tmp)
                *tmp = '\0';
              tmp = strchr(buf, '\r');
              if(tmp)
                *tmp = '\0';

              if(add_badword(client_p, chptr, buf) != 0)
                {
                  sendto_channel_local(ONLY_SERVERS, chptr, ":%s!%s@%s MODE %s :+G %s", source_p->name, source_p->username, source_p->host, chptr->chname, buf);
                  sendto_server(NULL, chptr, CAP_BAD, NOCAPS, ":%s!%s@%s MODE %s :+G %s", source_p->name, source_p->username, source_p->host, chptr->chname, buf);

                  if(words[0] == '\0')
                    strlcpy(words, buf, sizeof(words));
                  else
                    {
                      strlcat(words, buf, sizeof(words));
                    }
                }
            }
        }
        if(words[0] != '\0')
          sendto_one(source_p, "%s :Added %s", chptr->chname, words);
        else
          sendto_one(source_p, "%s :No Words Added", chptr->chname);

      fclose(fp);
    }

  sendto_channel_local(ONLY_CHANOPS, chptr, ":%s NOTICE %s :%s Used GRATED to reset filter.", "ChanFilter", chptr->chname, source_p->name);
  sendto_channel_remote(source_p, client_p, ONLY_CHANOPS, CAP_BAD|CAP_CHW, NOCAPS, chptr, ":%s NOTICE @%s :%s Used GRATED to reset filter.", source_p->name, chptr->chname, source_p->name);
}
  

/* END OF EXAMPLE MODULE */
