/*
 *  ircd-ratbox: an advanced Internet Relay Chat Daemon(ircd).
 *  channel.c: Controls channels.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center 
 * Copyright (C) 1996-2002 Hybrid Development Team 
 * Copyright (C) 2002 ircd-ratbox development team 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: autojoin.c,v 1.4 2002/12/12 05:31:54 jeremy Exp $
 */

#include "stdinc.h"
#include "tools.h"
#include "s_user.h"
#include "channel.h"
#include "channel_mode.h"
#include "class.h"
#include "client.h"
#include "common.h"
#include "fdlist.h"
#include "hash.h"
#include "irc_string.h"
#include "sprintf_irc.h"
#include "ircd.h"
#include "listener.h"
#include "motd.h"
#include "ircd_handler.h"
#include "msg.h"
#include "numeric.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "s_stats.h"
#include "scache.h"
#include "send.h"
#include "supported.h"
#include "whowas.h"
#include "md5.h"
#include "memory.h"
#include "packet.h"

struct autojoin_info AutojoinInfo;

struct ConfItem* aconf;
struct Channel *chptr;

int type;
char mode, sjmode, *newch, topic_info[USERHOST_REPLYLEN];

void autojoin(struct Client *);

dlink_node *ptr;

void
autojoin (struct Client *source_p)
{

ptr = source_p->localClient->confs.head;
aconf = ptr->data;

if(!IsConfDoAutoJoin(aconf))
  return;

if((AutojoinInfo.join_voiced) && !(AutojoinInfo.join_oped))
  {
    type = CHFL_VOICE;
    mode = 'v';
    sjmode = '+';
  }
else if(AutojoinInfo.join_oped)
  {
    type = CHFL_CHANOP;
    mode = 'o';
    sjmode = '@';
  }
else
  {
    type = CHFL_PEON;
    mode = sjmode = '\0';
  }
  if((chptr = hash_find_channel(AutojoinInfo.channel)) != NULL)
    {
      if(IsMember(source_p, chptr))
      {
        /* debugging is fun... */
        sendto_realops_flags(UMODE_DEBUG,L_ALL, "[AutoJoin] %s is already in %s", source_p->name, chptr->chname);
        ilog(L_NOTICE, "[AutoJoin] %s is already in %s", source_p->name, chptr->chname);
        return;
      }

      if(is_banned(chptr, source_p) == CHFL_BAN)
        {
           sendto_one(source_p, form_str(ERR_BANNEDFROMCHAN), source_p->name, me.name, chptr->chname);
           return;
        }

      add_user_to_channel(chptr, source_p, type);

        sendto_server(source_p, chptr, NOCAPS, NOCAPS,
                      ":%s SJOIN %lu %s + :%c%s",
                      me.name, (unsigned long) chptr->channelts,
                      chptr->chname, type ? sjmode : ' ', source_p->name);

      sendto_channel_local(ALL_MEMBERS, chptr, ":%s!%s@%s JOIN :%s",
                             source_p->name, source_p->username,
                             source_p->host, chptr->chname);

      if(type)
        sendto_channel_local(ALL_MEMBERS, chptr, ":%s MODE %s +%c %s",
                             me.name, chptr->chname, mode, source_p->name);
  
      if(chptr->topic != NULL)
      {
        sendto_one(source_p, form_str(RPL_TOPIC), me.name,
                   source_p->name, chptr->chname, chptr->topic);
        sendto_one(source_p, form_str(RPL_TOPICWHOTIME),
                   me.name, source_p->name, chptr->chname,
                   chptr->topic_info, chptr->topic_time);
      }

      channel_member_names(source_p, chptr, chptr->chname, 1);
    }
  else
    {
      newch = AutojoinInfo.channel;
      if (!check_channel_name(newch))
      {
        sendto_realops_flags(UMODE_DEBUG, L_ALL, "[AutoJoin] Bad Channel Name (%s)", AutojoinInfo.channel);
        ilog(L_CRIT, "[AutoJoin] Bad Channel Name (%s)", AutojoinInfo.channel);
        return;
      }

      /* channel name must begin with & or # */
      if (!IsChannelName(newch))
      {
        sendto_realops_flags(UMODE_DEBUG, L_ALL, "[AutoJoin] Bad Channel Name (%s)", AutojoinInfo.channel);
        ilog(L_NOTICE, "[AutoJoin] Bad Channel Name (%s)", AutojoinInfo.channel);
        return;
      }
      if (ConfigServerHide.disable_local_channels &&
          (*newch == '&'))
      {
        sendto_realops_flags(UMODE_DEBUG, L_ALL, "[AutoJoin] No Such Channel (%s)", newch);
        ilog(L_CRIT, "[AutoJoin] Bad Channel Name (%s). HINT: Local channels disabled.", AutojoinInfo.channel);
        return;
      }

      /* newch can't be longer than CHANNELLEN */
      if (strlen(newch) > CHANNELLEN)
      {
        sendto_realops_flags(UMODE_DEBUG, L_ALL, "[AutoJoin] Channel name is too long (%s)", AutojoinInfo.channel);
        ilog(L_CRIT, "[AutoJoin] Channel name is too long (%s)", AutojoinInfo.channel);
        return;
      }

      chptr = get_or_create_channel(source_p, newch, NULL);

        add_user_to_channel(chptr, source_p, type);

      /* send out a join, make target_p join chptr */
        sendto_server(source_p, chptr, NOCAPS, NOCAPS,
                      ":%s SJOIN %lu %s +nt :%s", me.name,
                      (unsigned long) chptr->channelts, chptr->chname,
                      source_p->name);

      sendto_channel_local(ALL_MEMBERS, chptr, ":%s!%s@%s JOIN :%s",
                             source_p->name, source_p->username,
                             source_p->host, chptr->chname);

      chptr->mode.mode |= MODE_TOPICLIMIT;
      chptr->mode.mode |= MODE_NOPRIVMSGS;

      sendto_channel_local(ALL_MEMBERS, chptr, ":%s MODE %s +nt", me.name,
                           chptr->chname);

      source_p->localClient->last_join_time = CurrentTime;
      channel_member_names(source_p, chptr, chptr->chname, 1);

      ircsprintf(topic_info, "%s", me.name);
      set_channel_topic(chptr, AutojoinInfo.topic, topic_info, CurrentTime);

      sendto_server(NULL, chptr, NOCAPS, NOCAPS,
                            ":%s TOPIC %s :%s",
                            me.name, chptr->chname,
                            chptr->topic == NULL ? "" : chptr->topic);
      sendto_channel_local(ALL_MEMBERS,
                                       chptr, ":%s TOPIC %s :%s",
                                       me.name,
                                       chptr->chname, chptr->topic == NULL ? "" : chptr->topic);     

      /* we do this to let the oper know that a channel was created, this will be
       * seen from the server handling the command instead of the server that
       * the oper is on.
       */
      ilog(L_NOTICE, "[AutoJoin] Creating channel %s", chptr->chname);
    }
   return;
}
